/**************************************************************************
					YarpBottleConversion.h
					-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#ifndef YARP_BOTTLE_CONVERSION_H
#define YARP_BOTTLE_CONVERSION_H

#include <rtt/TemplateTypeInfo.hpp>
#include "YarpTypeTemplateConversion.h"
#include <yarp/os/Vocab.h>

namespace OroYarp {

	/**
	 * Conversion structure for Yarp Bottles.
	 * Just make a copy of the bottle.
	 */
	template <>
	struct StdYarpTypeConversion<yarp::os::Bottle> : public YarpTypeConversion<yarp::os::Bottle, yarp::os::Bottle> {
		static bool copyYarpToOrocos(const yarp::os::Bottle& yarp, yarp::os::Bottle& orocos){
			orocos = yarp;
			return false;
		};
		static bool copyOrocosToYarp(const yarp::os::Bottle& orocos, yarp::os::Bottle& yarp){
			yarp = orocos;
			return true;
		};
	};
	
	/**
 	 * Wrapping Yarp bottles to Orocos.
	 */
	struct BottleTypeInfo : public RTT::TemplateTypeInfo<yarp::os::Bottle> {
		/** Wrapper constructor. */
		BottleTypeInfo() : RTT::TemplateTypeInfo<yarp::os::Bottle>("bottle") {};
		/**
		 * Converts a Yarp Bottle to an XML property.
		 * @param in The input bottle.
		 * @param out The output Property object.
		 * @return true if the conversion has been done.
		 */
		virtual bool decomposeTypeImpl(const yarp::os::Bottle& in, RTT::PropertyBag& out) const {
			Logger::In("BottleTypeInfo");
			out.setType("bottle");
			for (int i = 0; i < in.size(); i++) {
				yarp::os::Value& v = in.get(i);
				std::stringstream si, sv; si << i;
				switch(v.getCode()) {
				case BOTTLE_TAG_INT:
					out.add(new RTT::Property<int>(si.str(), "INT", v.asInt()));
					break;
				case BOTTLE_TAG_DOUBLE:
					out.add(new RTT::Property<double>(si.str(), "DOUBLE", v.asDouble()));
					break;
				case BOTTLE_TAG_STRING:
					out.add(new RTT::Property<std::string>(si.str(), "STRING", v.asString().c_str()));
					break;
				case BOTTLE_TAG_BLOB:
					log(Warning) << "Blobs are not decomposable..." << Logger::endl;
					break;
				case BOTTLE_TAG_VOCAB:
					sv << yarp::os::Vocab::decode(v.asVocab()).c_str();
					out.add(new RTT::Property<std::string>(si.str(), "VOCAB", sv.str()));
					break;
				default:
					if (v.isList()) {
						RTT::PropertyBag bag;
						decomposeTypeImpl(*(v.asList()), bag);
						out.add(new RTT::Property<RTT::PropertyBag>(si.str(), "LIST", bag));				
					}
				}
			}
			return true;
		};

		/**
		 * Converts an XML property to a Yarp Bottle.
		 * @param in The input Property object.
		 * @param out The output bottle.
		 * @return true if the conversion has been done.
		 */
		virtual bool composeTypeImpl(const RTT::PropertyBag& in, yarp::os::Bottle& out) const {
			Logger::In("BottleTypeInfo");
			if (in.getType().compare("bottle") == 0 || in.getType().compare("PropertyBag") == 0) {
				RTT::PropertyBag::Properties props = in.getProperties();
				for (RTT::PropertyBag::Properties::iterator i = props.begin(); i != props.end(); i++) {
					RTT::PropertyBase* p = *i;
					if (p->getType() == "int") {
						RTT::Property<int>* pi = RTT::Property<int>::narrow(p);
						out.addInt(pi->value());
					}
					if (p->getType() == "double") {
						RTT::Property<double>* pd = RTT::Property<double>::narrow(p);
						out.addDouble(pd->value());
					}
					if (p->getType() == "string") {
						RTT::Property<std::string>* ps = RTT::Property<std::string>::narrow(p);
						out.addString(ps->value().c_str());
					}
					if (p->getType() == "bottle" || p->getType() == "PropertyBag") {
						yarp::os::Bottle& bag = out.addList();
						RTT::Property<RTT::PropertyBag>* pp = RTT::Property<RTT::PropertyBag>::narrow(p);
						composeTypeImpl(*pp, bag);
					}
				}
				return true;
			}
			return false;
		};
		
		/**
		 * Displays Bottle.
		 * @param os The output stream.
		 * @param b The bottle.
		 * @return The output stream.
		 */
		virtual std::ostream& write(std::ostream &os, const yarp::os::Bottle& b) const {
			os << b.toString().c_str();
			return os;
		};

		/** 
		 * Creates bottle from stream.
		 * @param is The input stream.
		 * @param b The bottle.
		 * @return The input stream.
		 */
		virtual std::istream& read(std::istream& is, yarp::os::Bottle& b) const {
			std::string s;
			is >> s;
			b.fromString(s.c_str());
			return is;
		};	
		
	};

}

#endif
