/**************************************************************************
						YarpDeploymentComponent.cpp
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/

#include "YarpDeploymentComponent.h"
#include <rtt/scripting/StateMachineTask.hpp>

namespace OroYarp {

	using namespace RTT;
	using namespace std;
	using namespace boost;

	ScriptingPort::ScriptingPort(TaskContext* ydep) : deployer(ydep) {
		useCallback();
	}
	
	bool ScriptingPort::open(const char* name) {
		stringstream sname;
		sname << "/" << deployer->getName() << "/" << name;
		return yarp::os::BufferedPort<yarp::os::Bottle>::open(sname.str().c_str());
	}
	
	RequestPort::RequestPort(TaskContext* ydep) : ScriptingPort(ydep) {}

	void ScriptingPort::onRead(yarp::os::Bottle& b) {
		Logger::In("ScriptingPort");
		log(Debug) << deployer->getName() << " received script " << b.toString() << Logger::endl;
		if (b.size() <= 0) {
			log(Warning) << "Bottle is empty!" << Logger::endl;
			return;
		}
		string req(b.get(0).asString());
		int r = deployer->scripting()->execute(req);
		if (r < 0)
			log(Error) << "Unable to parse script '" << req << "'" << Logger::endl;
		if (r == 0)
			log(Info) << "Script code executed." << Logger::endl;
		else
			log(Info) << "Command being executing." << Logger::endl;
	}

	void RequestPort::onRead(yarp::os::Bottle& b) {
		Logger::In("RequestPort");
		log(Info) << deployer->getName() << " received request " << b.toString() << Logger::endl;
		if (b.size() <= 0) {
			log(Warning) << "Bottle is empty!" << Logger::endl;
			return;
		}
		string req;
		if (b.size() == 1) req = b.get(0).asString();
		else req = b.toString();
		// Look for space character.
		size_t space = req.find(" ");
		// Construct Request.
		string request, args;
		if (space == string::npos) {
			request = req;
			args = "";
		}
		else {
			request = req.substr(0, space);
			args = req.substr(space+1);
		}
		log(Info) << "Request: " << request << " ; " << args << Logger::endl;
		// Look for request FSM
		if (! deployer->scripting()->hasStateMachine(request)) {
			log(Error) << "No loaded State Machine named " << request << Logger::endl;
			return;
		}
		StateMachineTask* fsm = dynamic_cast<StateMachineTask*>(deployer->getObject(request));
		if (!fsm) {
			log(Error) << "Unable to access State Machine named " << request << Logger::endl;
			return;
		}
		// If FSM already running, do not accept request
		if (fsm->getStateMachine()->isAutomatic()) {
			log(Warning) << "Unable to launch FSM " << request << ": FSM is already running!" << Logger::endl;
			return;
		}
		// Activate FSM
		if (! fsm->getStateMachine()->isActive()) {
			if (fsm->getStateMachine()->activate()) {
				log(Info) << "Activating FSM " << request << Logger::endl;
			}
			else {
				log(Error) << "Unable to activate FSM " << request << Logger::endl;
				return;
			}
		}
		// Set FSM parameters
		AttributeRepository::AttributeNames params = fsm->attributes()->getAttributes();
		stringstream is(args);
		for (AttributeRepository::AttributeNames::const_iterator i = params.begin(); i != params.end(); i++) {
			DataSourceBase::shared_ptr data = fsm->attributes()->getValue(*i)->getDataSource();
			log(Info) << "Setting FSM parameter " << *i << " from " << is.str() << Logger::endl;
			try {
				data->getTypeInfo()->read(is, data);
			} catch (std::exception& e) {
				log(Warning) << "FSM parameter update raised exception " << e.what() << Logger::endl;
				break;
			} catch (...) {
				log(Warning) << "FSM parameter update raised an unknown exception" << Logger::endl;
				break;
			}
		}
		// Starts
		log(Info) << "Starting FSM " << request << Logger::endl;
		if (! fsm->getStateMachine()->start()) {
			log(Error) << "Unable to start FSM " << request << Logger::endl;
			return;
		}
	}
	
	YarpDeploymentComponent::YarpDeploymentComponent(const string& name) : DeploymentComponent(name),
		sport(this), rport(this) {
		// Methods
		this->methods()->addMethod(method("yarpnode", &YarpDeploymentComponent::createYarpNode, this),
			"Creates a YarpNode exporting the dataports for the given component",
			"tc", "Name of the TaskContext (must be a peer).");
		bool local_mode = yarp::os::Network::setLocalMode(false);
		if (local_mode)
			yarp::os::Network::setLocalMode(true);
		else if (!yarp::os::Network::checkNetwork())
			log(Error) << "Yarp server not found!" << Logger::endl;
		else {
			sport.open("Scripting");
			rport.open("Request");
		}
	}
	
	YarpDeploymentComponent::~YarpDeploymentComponent() {
		log(Debug) << "Deleting all Yarp nodes" << Logger::endl;
		for (unsigned int i = 0; i < yarpnodes.size(); i++)
			delete yarpnodes[i].yarp_;
	}
	
	bool YarpDeploymentComponent::createYarpNode(const std::string& tc) {
		Logger::In(this->getName());
		log(Debug) << "Trying to create YarpNode for component " << tc << Logger::endl;
		// Checking yarp server access
		if (!yarp::os::Network::checkNetwork()) {
			log(Error) << "Yarp server not found!" << Logger::endl;
			return false;
		}
		// Creating yarp node
		TaskContext* peer = this->getPeer(tc);
		if (!peer) {
			log(Error) << "No such peer: " << tc << Logger::endl;
			return false;
		}
		TaskContext* yarpnode = YarpNode::createYarpNode(peer, this->getName());
		if (yarpnode != 0) {
			this->addPeer(yarpnode);
			yarpnodes.push_back(couple(peer, yarpnode));
			connectPorts(peer->getName(), yarpnode->getName());
			return yarpnode->start();
		}
		return false;
	}

	bool YarpDeploymentComponent::componentLoaded(TaskContext* c) {
		bool yarp = comps[c->getName()].server;
		if (yarp) {
			log(Debug) << "Server flag was true" << Logger::endl;
			TaskContext* yarpnode = YarpNode::createYarpNode(c, this->getName());
			if (yarpnode != 0) {
				this->addPeer(yarpnode);
				yarpnodes.push_back(couple(c, yarpnode));
				return true;
			}
			return false;
		}
		return true;
	}

	bool YarpDeploymentComponent::connectYarpPorts(){
		bool return_ = true;
		for (unsigned int i = 0; i < yarpnodes.size(); i++) {
			return_ &= connectPorts(yarpnodes[i].rtt_->getName(), yarpnodes[i].yarp_->getName());
			yarpnodes[i].yarp_->start();
		}
		return return_;
	}

}
