/**************************************************************************
						YarpDeploymentComponent.h
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/

#ifndef YARP_DEPLOYMENT_COMPONENT_H
#define YARP_DEPLOYMENT_COMPONENT_H

#include <ocl/DeploymentComponent.hpp>
#include "Orocos2Yarp.h"

namespace OroYarp {
    
    /** Deployer Scripting Port class. */
	class ScriptingPort : public yarp::os::BufferedPort<yarp::os::Bottle> {
	protected:
		/** A pointer to the Yarp Deployer. */
		TaskContext* deployer;
   	public:
		/**
		 * The port constructor.
		 * @param deployer The Yarp deployer component.
		 */
		ScriptingPort(TaskContext* deployer);
		/**
		 * Parses the incoming Yarp bottle and launches the requested FSM with given parameters.
		 * @param b The Yarp bottle.
		 */
		virtual void onRead(yarp::os::Bottle& b);
		/**
		 * Open Yarp port.
		 * @param name The port name is "/DeployerName/name".
		 * @return Openning success status.
		 */
		virtual bool open(const char *name);
	};
 
    /** Deployer Supervising Port class. */
	class RequestPort : public ScriptingPort {
   	public:
		/**
		 * The port constructor.
		 * @param deployer The Yarp deployer component.
		 */
		RequestPort(TaskContext* deployer);
		/**
		 * Parses the incoming Yarp bottle and launches the requested FSM with given parameters.
		 * @param b The Yarp bottle.
		 */
		virtual void onRead(yarp::os::Bottle& b);
	};
    
    /**  Yarp Deployment Component. */
	class YarpDeploymentComponent : public OCL::DeploymentComponent {
	
		public:
			/**
			 * The Yarp Deployer constructor.
			 * @param name The Yarp Deployer name. Default is 'Deployer'.
			 */
			YarpDeploymentComponent(const std::string& name = "Deployer");
			/** The Yarp Deployer destructor. */
			~YarpDeploymentComponent();
			/**
			 * Connects Orocos ports duplicated to be binded to Yarp.
			 * @return true if all ports have been connected.
			 */
			bool connectYarpPorts();
			/**
			 * Creates a Yarp node associated to an Orocos component.
			 * @param tc The Orocos TaskContext.
			 * @return true if the Yarp node has been created.
			 */
			bool createYarpNode(const std::string& tc);

	    protected:
	    	/**
	    	 * Creates a Yarp node for a loaded component if its Server flag is true.
	    	 * @param c The loaded component.
	    	 * @return true if Yarp node creating succeeded.
	    	 */
			virtual bool componentLoaded(TaskContext* c);
			
		private:
		    /** Yarp Scripting port. */
			ScriptingPort sport;
		    /** Yarp Request port. */
			RequestPort rport;
			/** Couple structure carrying the Orocos component pointer and its corresponding OroYarp component pointer. */
			struct couple {
				/** The original Orocos component. */
				TaskContext* rtt_;
				/** The corresponding OroYarp component. */
				TaskContext* yarp_;
				/**
				 * Couple constructor.
				 * @param rtt The original Orocos component.
				 * @param yarp The OroYarp component.
				 */
				couple(TaskContext* rtt, TaskContext* yarp) : rtt_(rtt), yarp_(yarp) {};
			};
			/** Set of couples for created Yarp nodes. */
			std::vector<couple> yarpnodes;
	};
	
}

#endif
