/**************************************************************************
						YarpKDLConversion.h
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#ifndef YARP_KDL_CONVERSION_H
#define YARP_KDL_CONVERSION_H

#include "YarpTypeTemplateConversion.h"
#include <kdl/frames.hpp>
#include <kdl/frames_io.hpp>

namespace OroYarp {

	/**
	 * Conversion structure for KDL::Vector.
	 * The Yarp bottle contains vector values as doubles (ie. v[0] v[1] v[2]).
	 */
	template <>
	struct StdYarpTypeConversion<KDL::Vector> : public YarpTypeConversion<KDL::Vector, yarp::os::Bottle> {
		static bool copyYarpToOrocos(const yarp::os::Bottle& yarp, KDL::Vector& orocos){
			if (yarp.size() != 3) return false;
			for (int i = 0; i < 3; i++)
				orocos[i] = yarp.get(i).asDouble();
			return true;
		};
		static bool copyOrocosToYarp(const KDL::Vector& orocos, yarp::os::Bottle& yarp){
			yarp.clear();
			for (int i = 0; i < 3; i++)
				yarp.addDouble(orocos[i]);
			return true;
		};
	};
	
	/**
	 * Conversion structure for KDL::Rotation.
	 * The Yarp bottle contains matrix values as doubles (ie. r(0,0) r(0,1) r(0,2) r(1,0) ...)
	 */
	template <>
	struct StdYarpTypeConversion<KDL::Rotation> : public YarpTypeConversion<KDL::Rotation, yarp::os::Bottle> {
		static bool copyYarpToOrocos(const yarp::os::Bottle& yarp, KDL::Rotation& orocos){
			if (yarp.size() != 9) return false;
			for (int i = 0; i < 3; i++)
				for (int j = 0; j < 3; j++)
					orocos(i, j) = yarp.get(3*i+j).asDouble();
			return true;
		};
		static bool copyOrocosToYarp(const KDL::Rotation& orocos, yarp::os::Bottle& yarp){
			yarp.clear();
			for (int i = 0; i < 3; i++)
				for (int j = 0; j < 3; j++)
					yarp.addDouble(orocos(i, j));
			return true;
		};
	};

	/**
	 * Conversion structure for KDL::Frame.
	 * The Yarp bottle contains frame rotation matrix followed by frame vector.
	 */
	template <>
	struct StdYarpTypeConversion<KDL::Frame> : public YarpTypeConversion<KDL::Frame, yarp::os::Bottle> {
		static bool copyYarpToOrocos(const yarp::os::Bottle& yarp, KDL::Frame& orocos){
			if (yarp.size() != 12) return false;
			// Rotation
			for (int i = 0; i < 3; i++)
				for (int j = 0; j < 3; j++)
					orocos.M(i, j) = yarp.get(3*i+j).asDouble();
			// Vector
			for (int i = 0; i < 3; i++)
				orocos.p[i] = yarp.get(9+i).asDouble();
			return true;
		};
		static bool copyOrocosToYarp(const KDL::Frame& orocos, yarp::os::Bottle& yarp){
			yarp.clear();
			// Frame
			for (int i = 0; i < 3; i++)
				for (int j = 0; j < 3; j++)
					yarp.addDouble(orocos.M(i, j));
			// Vector
			for (int i = 0; i < 3; i++)
				yarp.addDouble(orocos.p[i]);
			return true;
		};
	};
	
	/**
	 * Conversion structure for KDL::Twist.
	 * The Yarp bottle contains twist values as doubles (ie. t[0] .. t[5]).
	 */
	template <>
	struct StdYarpTypeConversion<KDL::Twist> : public YarpTypeConversion<KDL::Twist, yarp::os::Bottle> {
		static bool copyYarpToOrocos(const yarp::os::Bottle& yarp, KDL::Twist& orocos){
			if (yarp.size() != 6) return false;
			for (int i = 0; i < 6; i++)
				orocos[i] = yarp.get(i).asDouble();
			return true;
		};
		static bool copyOrocosToYarp(const KDL::Twist& orocos, yarp::os::Bottle& yarp){
			yarp.clear();
			for (int i = 0; i < 6; i++)
				yarp.addDouble(orocos[i]);
			return true;
		};
	};
	
	/**
	 * Conversion structure for KDL::Wrench.
	 * The Yarp bottle contains wrench values as doubles (ie. w[0] .. w[5]).
	 */
	template <>
	struct StdYarpTypeConversion<KDL::Wrench> : public YarpTypeConversion<KDL::Wrench, yarp::os::Bottle> {
		static bool copyYarpToOrocos(const yarp::os::Bottle& yarp, KDL::Wrench& orocos){
			if (yarp.size() != 6) return false;
			for (int i = 0; i < 6; i++)
				orocos[i] = yarp.get(i).asDouble();
			return true;
		};
		static bool copyOrocosToYarp(const KDL::Wrench& orocos, yarp::os::Bottle& yarp){
			yarp.clear();
			for (int i = 0; i < 6; i++)
				yarp.addDouble(orocos[i]);
			return true;
		};
	};

}

#endif
