/**************************************************************************
						YarpPortCreator.h
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2010 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/

#ifndef YARP_PORT_CREATOR_H
#define YARP_PORT_CREATOR_H

#include "YarpReadPort.h"
#include "YarpWritePort.h"
#include "YarpRWPort.h"

namespace OroYarp {

	/** The global Yarp port creator. */
	class YarpPortCreator {
	private:
		/** The unique instance of Yarp port creator. */
		static YarpPortCreator* instance;
		/** The WritePortCreators type. */
		typedef std::map<std::string, boost::function<yarp::os::Port*(std::string, PortInterface*, TaskContext*)> > WritePortCreators;
		/** The set of Write Port creators available. */
		WritePortCreators write_port_creators_;
		/** The ReadPortCreators type. */
		typedef std::map<std::string, boost::function<yarp::os::Port*(std::string, PortInterface*)> > ReadPortCreators;
		/** The set of Read Port creators available. */
		ReadPortCreators read_port_creators_;
		/** The RWPortCreators type. */
		typedef std::map<std::string, boost::function<yarp::os::Port*(std::string, PortInterface*, TaskContext*)> > RWPortCreators;
		/** The set of RW Port creators available. */
		RWPortCreators rw_port_creators_;
	public:
		/**
		 * Creates a Write Yarp port exporting data from an Orocos port of component tc.
		 * @param node_name The Yarp node name.
		 * @param port The Orocos port.
		 * @param tc The Orocos component.
		 * @return A pointer to the Yarp port.
		 */
		yarp::os::Port* createWritePort(std::string node_name, PortInterface* port, TaskContext* tc) {
			const std::string& type_name = port->getTypeInfo()->getTypeName();
			WritePortCreators::iterator it = write_port_creators_.find(type_name);
			if (it!=write_port_creators_.end())
				return it->second(node_name, port, tc); 
			else {
			log(Error) << "Could not find a YarpWritePortCreator for type: " << type_name << Logger::endl;
				return 0;
			}
		};
		/**
		 * Creates a Read Yarp port importing data to an Orocos port.
		 * @param node_name The Yarp node name.
		 * @param port The Orocos port.
		 * @return A pointer to the Yarp port.
		 */
		yarp::os::Port* createReadPort(std::string node_name, PortInterface* port) {
			const std::string& type_name = port->getTypeInfo()->getTypeName();
			ReadPortCreators::iterator it = read_port_creators_.find(type_name);
			if (it != read_port_creators_.end())
				return it->second(node_name, port); 
			else {
				log(Error) << "Could not find a YarpReadPortCreator for type: " << type_name << Logger::endl;
				return 0;
			}
		};
		/**
		 * Creates a ReadWrite Yarp port.
		 * @param node_name The Yarp node name.
		 * @param port The Orocos port.
		 * @param tc The Orocos component.
		 * @return A pointer to the Yarp port.
		 */
		yarp::os::Port* createRWPort(std::string node_name, PortInterface* port, TaskContext* tc) {
			const std::string& type_name = port->getTypeInfo()->getTypeName();
			RWPortCreators::iterator it = rw_port_creators_.find(type_name);
			if (it!=rw_port_creators_.end())
				return it->second(node_name, port, tc); 
			else {
			log(Error) << "Could not find a YarpReadWritePortCreator for type: " << type_name << Logger::endl;
				return 0;
			}
		};
		/**
		 * Adds a new Yarp port creator to this instance.
		 * @param type_name The Orocos type that is now binded to Yarp.
		 * @return true if Read and Write Yarp ports have been found for this type.
		 */
		template <typename T>
		bool registerType(const std::string& type_name) {
			std::pair<WritePortCreators::iterator, bool> result1 =
				YarpPortCreator::Instance()->write_port_creators_.insert(WritePortCreators::value_type(type_name, &TemplateYarpWritePort<T>::createYarpWritePort));
			std::pair<ReadPortCreators::iterator, bool> result2 = 
				YarpPortCreator::Instance()->read_port_creators_.insert(ReadPortCreators::value_type(type_name, &TemplateYarpReadPort<T>::createYarpReadPort));
			std::pair<RWPortCreators::iterator, bool> result3 =
				YarpPortCreator::Instance()->rw_port_creators_.insert(RWPortCreators::value_type(type_name, &TemplateYarpReadWritePort<T>::createYarpReadWritePort));
			return result1.second && result2.second && result3.second;
		};
		/**
		 * Accessor to the Yarp port creator instance.
		 * @return A pointer to the instance.
		 */
		static YarpPortCreator* Instance() {
			if (instance == 0)
				instance = new YarpPortCreator();
			return instance;
		};
		/** Yarp port creator destructor. Deletes the instance. */
		~YarpPortCreator() {
			if (instance != 0) delete instance;
		};
		
	};
	
}

#endif
