/**************************************************************************
						YarpRWPort.h
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2010 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/

#ifndef YARP_RW_PORT_H
#define YARP_RW_PORT_H

#include "YarpTypeTemplateConversion.h"

namespace OroYarp {

	/** A Yarp port reading and writing data. */
	template <typename T>
	class TemplateYarpReadWritePort : public yarp::os::Port {
	private:
		/** The Yarp data type. */
		typedef typename StdYarpTypeConversion<T>::YarpType YT;
		/** The event handler corresponding to the Orocos port. */
		Handle event_handler_;
		/** The Orocos data source carrying the data. */
		ValueDataSource<T> data_;
		/** The Yarp object. */
		YT bottle_;
		/** The event received when a new data must be exported to Yarp. */
		typename DataPort<T>::NewDataOnPortEvent* event_;
		/** The object actually in charge of data conversion. */
		class DataProcessor : public yarp::os::PortReader {
		private:
			/** The Yarp received object. */
			YT b;
			/** The Orocos port where to put data. */
			const PortInterface* port_;
			/** The Orocos object. */
			T raw_;
			/** The Orocos data source carrying the object. */
			ValueDataSource<T> data_;
		public:
			/**
			 * Processor constructor.
			 * @param port The Orocos port to which it is binded.
			 */
			DataProcessor(PortInterface* port) : port_(port) {};
			/**
			 * Reads data from Yarp, converts it to Orocos type and put it in the Orocos port.
			 * @param connection The Yarp connection object.
			 * @return false if nothing has been read from Yarp connection.
			 */
			virtual bool read(yarp::os::ConnectionReader& connection) {
				//log(Debug) << "Copying data for port " << port_->getName() << Logger::endl;
				if (!b.read(connection)) {
					log(Debug) << "Impossible to read data from " << port_->getName() << Logger::endl;
					return false;
				}
				StdYarpTypeConversion<T>::copyYarpToOrocos(b, raw_);
				data_.set(raw_);
				if (port_->connected()) {
					log(Debug) << port_->getName() << " update port connection with datasource " << &raw_ << Logger::endl;
					port_->connection()->getDataSource()->update(data_.clone());
					port_->connection()->signal();
				} else
					log(Error) << port_->getName() << " has lost its connection :(" << Logger::endl;
				return true;
			};
		};
		/** The processor corresponding to this port. */
		DataProcessor processor;
	public:
		/**
		 * Creates a Yarp port associated to an Orocos port.
		 * @param yarp_node The Yarp node name.
		 * @param port The Orocos port.
		 * @param tc The TaskContext owning this port.
		 * @return A pointer to the corresponding Yarp port.
		 */
		static yarp::os::Port* createYarpReadWritePort(std::string yarp_node, PortInterface* port, TaskContext* tc) {
			return new TemplateYarpReadWritePort(yarp_node, port, tc);
		};
		/** YarpReadPort destructor. */
		~TemplateYarpReadWritePort() {
			this->close();
		};
	private:
		/**
		 * Yarp port constructor.
		 * @param node_name Yarp node name.
		 * @param port The Orocos port.
		 * @param tc The TaskContext owning this port.
		 */
		TemplateYarpReadWritePort(std::string node_name, PortInterface* port, TaskContext* tc) : processor(port) {
			log(Debug) << "Creating Yarp port " << port->getName() << endlog();
			this->open(std::string(node_name + "/" + port->getName()).c_str());
			this->setReader(processor);
			event_ = port->getNewDataOnPortEvent();
			this->event_handler_ = event_->connect(boost::bind(&TemplateYarpReadWritePort<T>::publishCallback, this, _1), 
				tc->engine()->events());
		};
		/**
		 * The callback function associated to the NewData event.
		 * @param port The Orocos port in which new data is to be exported.
		 */
		void publishCallback(PortInterface* port) {
			// Copy the data from the Orocos DataType to the Yarp port
			if (data_.update(port->connection()->getDataSource().get())) {
				//log(Debug) << "Publishing on port " << port->getName() << Logger::endl;
				StdYarpTypeConversion<T>::copyOrocosToYarp(data_.get(), this->bottle_);
				//Publish the message
				this->write(bottle_);
			} else
				log(Error) << "Could not update the data from " << port->getName() << Logger::endl;
		};

	};
	
}

#endif
