/**************************************************************************
						YarpTypeConversion.h
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#ifndef YARP_TYPE_CONVERSION_H
#define YARP_TYPE_CONVERSION_H

#include <rtt/TemplateTypeInfo.hpp>
#include "YarpTypeTemplateConversion.h"

namespace OroYarp {

	using namespace RTT;

	/**
	 * Conversion structure for doubles.
	 * The Yarp bottle contains the double value.
	 */
	template <>
	struct StdYarpTypeConversion<double> : public YarpTypeConversion<double, yarp::os::Bottle> {
		static bool copyYarpToOrocos(const YarpType& yarp, OrocosType& orocos){
			for (int i = 0; i < yarp.size(); i++)
				if (yarp.get(i).isDouble() || yarp.get(i).isInt()) {
					orocos = yarp.get(i).asDouble();
					return true;
				}
			return false;
		};
		static bool copyOrocosToYarp(const OrocosType& orocos, YarpType& yarp){
			yarp.clear();
			yarp.addDouble(orocos);
			return true;
		};
	};
	
	/**
	 * Conversion structure for vectors of doubles.
	 * The Yarp bottle contains all vector values in the same order as the vector.
	 */
	template <>
	struct StdYarpTypeConversion<std::vector<double> > : public YarpTypeConversion<std::vector<double>, yarp::os::Bottle> {
		static bool copyYarpToOrocos(const YarpType& yarp, OrocosType& orocos) {
			if (yarp.size() == 0) return false;
			orocos.clear();
			// If bottle contains a list
			if (yarp.get(0).isList()) {
				yarp::os::Bottle* l = yarp.get(0).asList();
				for (int i = 0; i < l->size(); i++)
					if (l->get(i).isDouble() || l->get(i).isInt())
						orocos.push_back(l->get(i).asDouble());
				return (orocos.size() != 0);
			}
			// If bottle contains values
			for (int i = 0; i < yarp.size(); i++)
				if (yarp.get(i).isDouble() || yarp.get(i).isInt())
					orocos.push_back(yarp.get(i).asDouble());
			return (orocos.size() != 0);
		};
		static bool copyOrocosToYarp(const OrocosType& orocos, YarpType& yarp){
			yarp.clear();
			for (std::vector<double>::const_iterator i = orocos.begin(); i != orocos.end(); i++)
				yarp.addDouble(*i);
			return true;
		};
	};
	
	/**
	 * Conversion structure for integers.
	 * The Yarp bottle contains the integer value.
	 */
	template <>
	struct StdYarpTypeConversion<int> : public YarpTypeConversion<int, yarp::os::Bottle> {
		static bool copyYarpToOrocos(const YarpType& yarp, OrocosType& orocos){
			for (int i = 0; i < yarp.size(); i++)
				if (yarp.get(i).isInt()) {
					orocos = yarp.get(i).asInt();
					return true;
				}
			return false;
		};
		static bool copyOrocosToYarp(const OrocosType& orocos, YarpType& yarp){
			yarp.clear();
			yarp.addInt(orocos);
			return true;
		};
	};
	
	/**
	 * Conversion structure for booleans.
	 * The Yarp bottle contains an integer value casted from the boolean.
	 */
	template <>
	struct StdYarpTypeConversion<bool> : public YarpTypeConversion<bool, yarp::os::Bottle> {
		static bool copyYarpToOrocos(const YarpType& yarp, OrocosType& orocos){
			for (int i = 0; i < yarp.size(); i++)
				if (yarp.get(i).isInt()) {
					orocos = (yarp.get(i).asInt() != 0);
					return true;
				}
			return false;
		};
		static bool copyOrocosToYarp(const OrocosType& orocos, YarpType& yarp){
			yarp.clear();
			yarp.addInt(orocos);
			return true;
		};
	};
	
	/**
	 * Conversion structure for strings.
	 * The Yarp bottle contains the string.
	 */
	template <>
	struct StdYarpTypeConversion<std::string> : public YarpTypeConversion<std::string, yarp::os::Bottle> {
		static bool copyYarpToOrocos(const YarpType& yarp, OrocosType& orocos){
			for (int i = 0; i < yarp.size(); i++)
				if (yarp.get(i).isString()) {
					orocos = yarp.get(i).asString();
					return true;
				}
			return false;
		};
		static bool copyOrocosToYarp(const OrocosType& orocos, YarpType& yarp){
			yarp.clear();
			yarp.addString(orocos.c_str());
			return true;
		};
	};

}

#endif
