/**************************************************************************
						YarpWritePort.h
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2010 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/

#ifndef YARP_WRITE_PORT_H
#define YARP_WRITE_PORT_H

#include "YarpTypeTemplateConversion.h"

namespace OroYarp {

	/** A Yarp port exporting data from an Orocos port to Yarp. */
	template <typename T>
	class TemplateYarpWritePort : public yarp::os::Port {
	private:
		/** The event handler corresponding to the Orocos port. */
		Handle event_handler_;
		/** The Orocos data source carrying the data. */
		ValueDataSource<T> data_;
		/** The corresponding Yarp type. */
		typedef typename StdYarpTypeConversion<T>::YarpType YT;
		/** The Yarp object. */
		YT bottle_;
		/** The event received when a new data must be exported to Yarp. */
		typename DataPort<T>::NewDataOnPortEvent* event_;
	public:
		/**
		 * Creates a Yarp port exporting data of an Orocos port.
		 * @param node_name The Yarp node name.
		 * @param port The Orocos port to export.
		 * @param tc The TaskContext owning this port.
		 * @return A pointer to the Yarp port.
		 */
		static yarp::os::Port* createYarpWritePort(std::string node_name, PortInterface* port, TaskContext* tc) {
			return new TemplateYarpWritePort(node_name, port, tc);
		};
		/** YarpWritePort destructor. */
		~TemplateYarpWritePort() {
			this->close();
		};
	private:
		/**
		 * Yarp port constructor.
		 * @param node_name Yarp node name.
		 * @param port The Orocos port to export.
		 * @param tc The TaskContext owning this port.
		 */
		TemplateYarpWritePort(std::string node_name, PortInterface* port, TaskContext* tc) {
			log(Debug) << "Creating Yarp port " << port->getName() << Logger::endl;
			this->open(std::string(node_name + "/" + port->getName()).c_str());
			event_ = port->getNewDataOnPortEvent();
			this->event_handler_ = event_->connect(boost::bind(&TemplateYarpWritePort<T>::publishCallback, this, _1), 
				tc->engine()->events());
		};
		/**
		 * The callback function associated to the NewData event.
		 * @param port The Orocos port in which new data is to be exported.
		 */
		void publishCallback(PortInterface* port) {
			// Copy the data from the Orocos DataType to the Yarp port
			if (data_.update(port->connection()->getDataSource().get())) {
				//log(Debug) << "Publishing on port " << port->getName() << Logger::endl;
				StdYarpTypeConversion<T>::copyOrocosToYarp(data_.get(), this->bottle_);
				//Publish the message
				this->write(bottle_);
			} else
				log(Error) << "Could not update the data from " << port->getName() << Logger::endl;
		};
		
	};

}

#endif
