/**************************************************************************
						test_yarp_request.cpp
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2010 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#include <rtt/TaskContexts.hpp>
#include <rtt/scripting/StateMachineTask.hpp>
#include <rtt/os/main.h>
#include <rtt/Toolkit.hpp>
#include <yarp/os/all.h>
#include "YarpDeploymentComponent.h"
#include "YarpToolkit.h"
#include <boost/progress.hpp>
#include <boost/thread.hpp>

yarp::os::Port p;
yarp::os::Bottle bottle;

void test_state(RTT::TaskContext* dc, const std::string& s) {
	boost::timer t;
	while(true) {
		boost::this_thread::sleep(boost::posix_time::milliseconds(1));
		if (dc->execution()->isStateMachineRunning("test")) {
			if (dc->execution()->inStateMachineState("test", s)) {
				std::cout << "OK (FSM 'test' is in state " << dc->scripting()->getCurrentState("test") << ")" << std::endl;
				break;
			}
			else {
				std::cout << "FAIL (FSM 'test' is in state " << dc->scripting()->getCurrentState("test") << ")" << std::endl;
				exit(-1);
			}
		}
	}
}

template <typename T>
void test_param(RTT::StateMachineTask* fsm, const std::string& param, const T& value) {
	RTT::Attribute<T>* v = fsm->attributes()->getAttribute<T>(param);
	assert(v);
	assert(v->get() == value);
}

int ORO_main(int argc, char** argv) {
	RTT::log().setLogLevel(RTT::Logger::Debug);
	srand(time(NULL));
	
	// Importing Yarp Toolkit
	std::cout << "Importing Orocos Yarp Toolkit... " << std::flush;
	RTT::Toolkit::Import(OroYarp::YarpToolkit);
	std::cout << "OK" << std::endl;
	
	// Initialize yarp network
	std::cout << "Initializing Yarp network... " << std::flush;
	yarp::os::Network::init();
	if (!yarp::os::Network::checkNetwork())
		yarp::os::Network::setLocalMode(true);
	assert(p.open("/yarp"));
	std::cout << "OK" << std::endl;
	
	// Creating Yarp Deployer
	std::cout << "Creating Yarp deployer... " << std::flush;
	OroYarp::YarpDeploymentComponent dc;
	dc.start();
	assert(yarp::os::Network::connect("/yarp", "/Deployer/Request"));
	std::cout << "OK" << std::endl;
	
	// Loading the test FSM
	std::cout << "Loading test FSM " 
		<< std::string(STATE_MACHINES_PATH) << "yarpscripting_fsm.osd... " << std::flush;
	assert(dc.scripting()->loadStateMachines(std::string(STATE_MACHINES_PATH) + "yarpscripting_fsm.osd"));
	std::cout << "OK" << std::endl;
	// Checking FSM status
	std::cout << "Checking FSM status... " << dc.scripting()->getStateMachineStatus("test") << std::endl;
	
	// Launching test FSM from Yarp
	std::cout << "Requesting for FSM 'test'..." << std::endl;
	int i = rand() % 100 + 1; 
	double d = rand() % 1000 / 10.; 
	bool b = bool(rand() % 2); 
	std::string s = "World";
	std::stringstream ss; ss << "test " << i << " " << d << " " << s << " " << std::boolalpha << b;
	bottle.fromString(ss.str().c_str());
	p.write(bottle);
	std::cout << "\tSent bottle '" << bottle.toString() << "' and waits for FSM execution..." << std::endl;
	test_state(&dc, "P");
	
	// Checking parameters
	std::cout << "Checking FSM parameters... " << std::flush;
	RTT::StateMachineTask* fsm = dynamic_cast<RTT::StateMachineTask*>(dc.getObject("test"));
	assert(fsm);
	test_param<int>(fsm, "V_int", i);
	test_param<double>(fsm, "V_double", d);
	test_param<std::string>(fsm, "V_string", s);
	test_param<bool>(fsm, "V_bool", b);
	std::cout << "OK" << std::endl;
	
	// Quit
    yarp::os::Network::fini();
    std::cout << "Test Successful." << std::endl;
	return 0;
}
