/**************************************************************************
						yarpnode_test.cpp
						-----------------------
	author			: Charles Lesire-Cabaniols
	copyright		: Copyright 2009 Onera
	email			: charles.lesire@onera.fr

 **************************************************************************
 * This file is part of RoboTIS OroYarp package.
 *
 * OroYarp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * OroYarp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with OroYarp.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
#include <rtt/TaskContexts.hpp>
#include <rtt/os/main.h>
#include <rtt/Toolkit.hpp>
#include <yarp/os/all.h>
#include "Orocos2Yarp.h"
#include "YarpToolkit.h"

yarp::os::Port p;
yarp::os::Bottle b;

template <typename T>
void sendYarpOrocos(T& v, const char* oroyarp, RTT::ReadDataPort<T>* oroport) {
	assert(yarp::os::Network::connect("/yarp", oroyarp));
	std::cout << "Sending '" << v << "' from Yarp to Orocos... " << std::flush;
	b.add(v);
	p.write(b);
	assert(oroport->Get() == v);
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/yarp", oroyarp);
}

template <typename T>
void sendOrocosYarp(T& v, const char* oroyarp, RTT::WriteDataPort<T>* oroport) {
	assert(yarp::os::Network::connect(oroyarp, "/yarp"));
	std::cout << "Sending '" << v << "' from Orocos to Yarp... " << std::flush;
	oroport->Set(v);
	p.read(b);
	assert(b.get(0) == yarp::os::Value(v));
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect(oroyarp, "/yarp");
}

int ORO_main(int argc, char** argv) {
	RTT::log().setLogLevel(RTT::Logger::Never);
	yarp::os::Network yarpN;
	srand(time(NULL));
	
	// Importing Yarp Toolkit
	std::cout << "Importing Orocos Yarp Toolkit... " << std::flush;
	RTT::Toolkit::Import(OroYarp::YarpToolkit);
	std::cout << "OK" << std::endl;
	
	// Initialize yarp network
	std::cout << "Initializing Yarp network... " << std::flush;
	yarp::os::Network::init();
	if (!yarp::os::Network::checkNetwork())
		yarp::os::Network::setLocalMode(true);
	assert(p.open("/yarp"));
	std::cout << "OK" << std::endl;

	// Create TestComponent
    std::cout << "Creating Test component... " << std::flush;
    RTT::TaskContext* test = new RTT::TaskContext("Test");
	RTT::ReadDataPort<int> in_int("in_int"); test->ports()->addPort(&in_int);
	RTT::WriteDataPort<int> out_int("out_int"); test->ports()->addPort(&out_int);
    RTT::ReadDataPort<double> in_double("in_double"); test->ports()->addPort(&in_double);
	RTT::WriteDataPort<double> out_double("out_double"); test->ports()->addPort(&out_double);
	RTT::ReadDataPort<bool> in_bool("in_bool"); test->ports()->addPort(&in_bool);
	RTT::WriteDataPort<bool> out_bool("out_bool"); test->ports()->addPort(&out_bool);
	RTT::ReadDataPort<std::string> in_string("in_string"); test->ports()->addPort(&in_string);
	RTT::WriteDataPort<std::string> out_string("out_string"); test->ports()->addPort(&out_string);
	RTT::ReadDataPort<std::vector<double> > in_vector("in_vector"); test->ports()->addPort(&in_vector);
	RTT::WriteDataPort<std::vector<double> > out_vector("out_vector"); test->ports()->addPort(&out_vector);
	std::cout << "OK" << std::endl;

	// Launching TestComponent
	std::cout << "Starting Test component... " << std::flush;
	assert(test->start());
	std::cout << "OK" << std::endl;
	
	// Binding to Yarp
	std::cout << "Binding Test component to Yarp... " << std::flush;
	RTT::TaskContext* yarpnode = OroYarp::YarpNode::createYarpNode(test);
	assert(yarpnode);
	assert(connectPorts(yarpnode, test));
	assert(yarpnode->start());
	std::cout << "OK" << std::endl;
	
	std::cout << "TESTING CONVERSION OF INT" << std::endl;
	// Yarp --> Orocos
	int i = rand() % 100;
	sendYarpOrocos<int>(i, "/Orocos/Test/in_int", &in_int);
	// Orocos --> Yarp
	i = rand() % 100;
	sendOrocosYarp<int>(i, "/Orocos/Test/out_int", &out_int);
	
	std::cout << "TESTING CONVERSION OF DOUBLE" << std::endl;
	// Yarp --> Orocos
	double d = (rand() % 1000) / 10.;
	sendYarpOrocos<double>(d , "/Orocos/Test/in_double", &in_double);
	// Orocos --> Yarp
	d = (rand() % 1000) / 10.;
	sendOrocosYarp<double>(d, "/Orocos/Test/out_double", &out_double);
	
	std::cout << "TESTING CONVERSION OF BOOL" << std::endl;
	// Yarp --> Orocos
	bool bo = bool(rand() % 2);
	sendYarpOrocos<bool>(bo , "/Orocos/Test/in_bool", &in_bool);
	// Orocos --> Yarp
	bo = (rand() % 2);
	sendOrocosYarp<bool>(bo, "/Orocos/Test/out_bool", &out_bool);
	
	std::cout << "TESTING CONVERSION OF STD::STRING" << std::endl;
	// Yarp --> Orocos
	std::string s("Hello");
	assert(yarp::os::Network::connect("/yarp", "/Orocos/Test/in_string"));
	std::cout << "Sending '" << s << "' from Yarp to Orocos... " << std::flush;
	b.addString(s.c_str());
	p.write(b);
	assert(in_string.Get() == s);
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/yarp", "/Orocos/Test/in_string");
	// Orocos --> Yarp
	s = "World";
	assert(yarp::os::Network::connect("/Orocos/Test/out_string", "/yarp"));
	std::cout << "Sending '" << s << "' from Orocos to Yarp... " << std::flush;
	out_string.Set(s);
	p.read(b);
	assert(b.get(0) == yarp::os::Value(s.c_str()));
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/Orocos/Test/out_string", "/yarp");
		
	std::cout << "TESTING CONVERSION OF STD::VECTOR" << std::endl;
	// Yarp -- vector --> Orocos
	std::vector<double> v(rand()%10 + 1);
	for (std::vector<double>::iterator it = v.begin(); it != v.end(); it++) *it = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/yarp", "/Orocos/Test/in_vector"));
	std::cout << "Sending '"; 
	for(std::vector<double>::iterator it=v.begin(); it!=v.end(); it++) std::cout << *it << " ";
	std::cout << "' from Yarp to Orocos... " << std::flush;
	for (std::vector<double>::iterator it = v.begin(); it != v.end(); it++) b.addDouble(*it);
	p.write(b);
	assert(in_vector.Get() == v);
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/yarp", "/Orocos/Test/in_vector");
	// Orocos -- vector --> Yarp
	v.clear(); v.resize(rand()%10 + 1);
	for (std::vector<double>::iterator it = v.begin(); it != v.end(); it++) *it = (rand() % 1000) / 10.;
	assert(yarp::os::Network::connect("/Orocos/Test/out_vector", "/yarp"));
	std::cout << "Sending '"; 
	for(std::vector<double>::iterator it=v.begin(); it!=v.end(); it++) std::cout << *it << " ";
	std::cout << "' from Yarp to Orocos... " << std::flush;
	out_vector.Set(v);
	p.read(b);
	for (int k = 0; k < b.size(); k++) assert(b.get(k).asDouble() == v.at(k));
	b.clear();
	std::cout << "OK" << std::endl;
	yarp::os::Network::disconnect("/Orocos/Test/out_vector", "/yarp");
		
	// Quit
	delete yarpnode;
	delete test;
    yarp::os::Network::fini();
    std::cout << "Test Successful." << std::endl;
	return 0;
}

